/**
 * Copyright (c) 2013-2017 Lorenzo Bettini.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Lorenzo Bettini - Initial contribution and API
 */
package org.eclipse.xsemantics.dsl.ui.contentassist;

import com.google.inject.Inject;
import java.util.List;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xsemantics.dsl.services.XsemanticsProposalsForDatatypeRules;
import org.eclipse.xsemantics.runtime.XsemanticsRuntimeSystem;
import org.eclipse.xtext.Assignment;
import org.eclipse.xtext.RuleCall;
import org.eclipse.xtext.common.types.JvmType;
import org.eclipse.xtext.common.types.TypesPackage;
import org.eclipse.xtext.common.types.access.IJvmTypeProvider;
import org.eclipse.xtext.common.types.xtext.ui.ITypesProposalProvider;
import org.eclipse.xtext.ui.editor.contentassist.ContentAssistContext;
import org.eclipse.xtext.ui.editor.contentassist.ICompletionProposalAcceptor;
import org.eclipse.xtext.validation.AbstractDeclarativeValidator;

/**
 * see http://www.eclipse.org/Xtext/documentation.html#contentAssist on how to customize content assistant
 */
@SuppressWarnings("all")
public class XsemanticsProposalProvider extends AbstractXsemanticsProposalProvider {
  @Inject
  private ITypesProposalProvider typeProposalProvider;

  @Inject
  private IJvmTypeProvider.Factory typeProviderFactory;

  @Inject
  private XsemanticsProposalsForDatatypeRules proposalsForDatatypeRules;

  @Override
  public void complete_JudgmentSymbol(final EObject model, final RuleCall ruleCall, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    List<String> _judgmentSymbols = this.proposalsForDatatypeRules.judgmentSymbols();
    for (final String symbol : _judgmentSymbols) {
      acceptor.accept(this.createCompletionProposal(symbol, context));
    }
  }

  @Override
  public void complete_RelationSymbol(final EObject model, final RuleCall ruleCall, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    List<String> _relationSymbols = this.proposalsForDatatypeRules.relationSymbols();
    for (final String symbol : _relationSymbols) {
      acceptor.accept(this.createCompletionProposal(symbol, context));
    }
  }

  @Override
  public void completeXsemanticsSystem_ValidatorExtends(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    this.showOnlySubtypesOf(model, context, acceptor, 
      AbstractDeclarativeValidator.class);
  }

  @Override
  public void completeXsemanticsSystem_SuperSystem(final EObject model, final Assignment assignment, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor) {
    this.showOnlySubtypesOf(model, context, acceptor, 
      XsemanticsRuntimeSystem.class);
  }

  private void showOnlySubtypesOf(final EObject model, final ContentAssistContext context, final ICompletionProposalAcceptor acceptor, final Class<?> superType) {
    final IJvmTypeProvider jvmTypeProvider = this.typeProviderFactory.createTypeProvider(model.eResource().getResourceSet());
    final JvmType interfaceToImplement = jvmTypeProvider.findTypeByName(superType.getName());
    this.typeProposalProvider.createSubTypeProposals(interfaceToImplement, this, context, 
      TypesPackage.Literals.JVM_PARAMETERIZED_TYPE_REFERENCE__TYPE, acceptor);
  }
}
